//Easy Socket Class

#include "MyNetwork.h"

MyNetwork::MyNetwork(char *_port)
{
	port = _port;
}

int MyNetwork::start(ptrFunc myFunc)
{

	int yes =1;

	//0 out the struct
	memset(&hints, 0, sizeof hints);

	//setup the type of socket
	//use IPV4
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;




		//load the structure as a server
		//use local machine address
		hints.ai_flags = AI_PASSIVE;

		//make sure addrinfo is good
		if(getaddrinfo( NULL, port, &hints, &result)!=0)
		{
			perror("getaddrinfo error \n");
			return 1;
		}


		//Setup the socket for future use, loop through to get the next available
		for(p = result; p != NULL; p = p->ai_next)
		{
			if((s = socket(result->ai_family, result->ai_socktype, result->ai_protocol))==-1)
				{
					perror("Error creating socket\n");
					continue;
				}

				//set socket options
				if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int))== -1)
				{
					perror ("Set sock opt\n");
					return 1;
				}
				//bind to the socket
				if(bind(s, result->ai_addr, result->ai_addrlen)==-1)
				{
					perror("error on bind\n");
					return 1;
				}
				break;

		}

		if(p == NULL)
		{
			perror("failed to bind to socket\n");
			return 1;
		}

		//free up the struct
		freeaddrinfo(result);

		if(listen(s,BACKLOG) == -1)
		{
			perror("error on listen\n");
			return 1;
		}

		sa.sa_handler = sigHandle;
		sigemptyset(&sa.sa_mask);
		sa.sa_flags = SA_RESTART;
		if(sigaction(SIGCHLD, &sa, NULL) == -1)
		{
			perror("Sig Error \n");
			exit(1);
		}

		std::cout<<"server Running\n";
		//run the accept loop
		acceptLoop(myFunc);




	return 0;
}

void *MyNetwork::get_in_addr(struct sockaddr *sa)
{
	if(sa->sa_family == AF_INET)
	{
		return &(((struct sockaddr_in*)sa)->sin_addr);
	}
	return &(((struct sockaddr_in6*)sa)->sin6_addr);
}

MyNetwork::~MyNetwork()
{
	//close the open socket
	close(s);
}

void MyNetwork::sendPacket(std::string* buff, int msgSize)
{
    std::string lBuff = *buff;
    msgSize+=1;//string.size() doesnt account for /0
    char* msg = new char[msgSize];
    strcpy(msg,lBuff.c_str());
	int byteSent = 0;
	int n = 0;
	int bytesLeft =  msgSize;
	//the send() doesn't always send all the bytes in one shot.
	//since it returns the bytes it sent you can used that to see what is left
	//and send the remaining bytes.
	while(byteSent<msgSize)
	{
		msg +=byteSent;
		n = send(newSock, msg, bytesLeft, 0);
		if(n == -1) {break;}//error has occured here
		byteSent += n;
		bytesLeft -= n;
	}

    delete msg;

}


void MyNetwork::sendPacket(const char *msg_)
{
    std::string lBuff = msg_;
    int msgSize = lBuff.length()+1;//string.size() doesnt account for /0
    char* msg = new char[msgSize];
    strcpy(msg,lBuff.c_str());
	int byteSent = 0;
	int n = 0;
	int bytesLeft =  msgSize;
	//the send() doesn't always send all the bytes in one shot.
	//since it returns the bytes it sent you can used that to see what is left
	//and send the remaining bytes.
	while(byteSent<msgSize)
	{
		msg +=byteSent;
		n = send(newSock, msg, bytesLeft, 0);
		if(n == -1) {break;}//error has occured here
		byteSent += n;
		bytesLeft -= n;
	}

    delete msg;

}

bool MyNetwork::charSearch(const char *buffer, const char* searchString)
{
	int len = strlen(buffer);
	int forLen = strlen(searchString); // The length of the searchfor field

	//Search through each char in toSearch
	for (int i = 0; i < len;i++)
	{
		//If the active char is equil to the first search item then search toSearch
		if (searchString[0] == buffer[i])
		{
			bool found = true;
			//search the char array for search field
			for (int x = 1; x < forLen; x++)
			{
				if (buffer[i+x]!=searchString[x])
				{
					found = false;
				}
			}

			//if found return true;
			if (found == true)
				return true;
		}
	}

	return 0;
}


char* MyNetwork::readPacket()
{
    clearBuff(); //clear the internal buffer
    char*p = &buff[0];//reset the pointer to the first element


    int n =0;
	//recv returns 0 when client has disconnected
	if((n=recv(newSock, p, buffLimit ,0))==0)
	{
		perror("client disconnected\n");
		return 0;
	}
	
	//the recv() fills the buffer using the pointer, return it for processing
	return p;
}


int MyNetwork::acceptLoop(ptrFunc myFunc)
{
	while(1)
	{
	addrSize = sizeof(theirAddr);
	newSock = accept(s,(struct sockaddr*)&theirAddr, &addrSize);
	if(newSock == -1)
	{
		perror("error on accept \n");
		continue;
	}

	inet_ntop(theirAddr.ss_family,
	get_in_addr((struct sockaddr*)&theirAddr),
	inAddr, sizeof(inAddr));
	// inAddr now equals the IP of the connected client
	//add logging here
	std::cout<<"incomming connection from: "<<inAddr<<std::endl;

	//create a child process
	if(!fork())
	{
		close(s); //child process doesn't need the listener
		int a = 0;
		//run the server handler function, pass the false variable
		myFunc(a);

		close(newSock);
		exit(0);
	}
	close(newSock); //parent doesn't need this anymore
	}
	return 0;
}



 void MyNetwork::sigHandle(int s)
{
	while(waitpid(-1,NULL, WNOHANG) >0);
}

void MyNetwork::clearBuff()
{
	 int cnt = 0;
	
	while(cnt< buffLimit)
	{
		buff[cnt++] ='\0';
		
	}	
}



